/*
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Bonneau and Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.debug.ui.model;

import org.eclipse.core.resources.IMarkerDelta;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugEvent;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.IBreakpointManager;
import org.eclipse.debug.core.IBreakpointManagerListener;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.model.IBreakpoint;
import org.eclipse.debug.core.model.IDebugTarget;
import org.eclipse.debug.core.model.IMemoryBlock;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.IThread;

import fr.obeo.acceleo.gen.debug.ui.AcceleoGenDebugUiPlugin;
import fr.obeo.acceleo.gen.template.debug.ITemplateDebugger;
import fr.obeo.acceleo.gen.template.debug.ITemplateDebuggerListener;

/**
 * Acceleo Debug Target.
 */
public class AcceleoDebugTarget extends AcceleoDebugElement implements IDebugTarget, IBreakpointManagerListener {

	private AcceleoProcess process;

	private AcceleoThread thread;

	private ILaunch launch;

	private ITemplateDebugger debugger;

	private ITemplateDebuggerListener debuggerListener;

	private boolean terminated = false;

	/**
	 * Constructor.
	 * 
	 * @param aLaunch
	 *            is the current launch configuration
	 * @param aTemplateDebugger
	 *            is the debugger
	 */
	public AcceleoDebugTarget(ILaunch aLaunch, ITemplateDebugger aTemplateDebugger) {
		super(null);
		launch = aLaunch;
		debugger = aTemplateDebugger;
		process = new AcceleoProcess(this);
		// Link beetween launch / process / thread
		launch.addProcess(process);
		thread = new AcceleoThread(this);
		// Listen for breakpoint events
		IBreakpointManager breakpointManager = getBreakpointManager();
		breakpointManager.addBreakpointListener(this);
		breakpointManager.addBreakpointManagerListener(this);

		IBreakpoint[] breakpoints = getBreakpointManager().getBreakpoints();
		// Init debuger with break points
		debugger.init();
		for (int i = 0; i < breakpoints.length; ++i) {
			if (breakpoints[i] instanceof AcceleoLineBreakpoint) {
				AcceleoLineBreakpoint lineBreakpoint = (AcceleoLineBreakpoint) breakpoints[i];
				try {
					if (lineBreakpoint.isEnabled()) {
						debugger.addBreakpoint(lineBreakpoint.getFile(), lineBreakpoint.getLineNumber());
					}
				} catch (CoreException e) {
					AcceleoGenDebugUiPlugin.getDefault().log(e, true);
				}
			}
		}

		debuggerListener = new ITemplateDebuggerListener() {

			public void resumeClient() {
				thread.setSuspended(false);
				thread.setStepping(false);
				thread.fireResumeEvent(DebugEvent.CLIENT_REQUEST);

			}

			public void suspendBreakpoint() {
				thread.setSuspended(true);
				thread.setStepping(false);
				thread.fireSuspendEvent(DebugEvent.BREAKPOINT);

			}

			public void resumeStep() {
				thread.setSuspended(false);
				thread.setStepping(true);
				thread.fireResumeEvent(DebugEvent.STEP_OVER);
			}

			public void suspendStep() {
				thread.setSuspended(true);
				thread.setStepping(false);
				thread.fireSuspendEvent(DebugEvent.STEP_END);
			}

			public void end() {
				try {
					terminate();
				} catch (DebugException e) {
					AcceleoGenDebugUiPlugin.getDefault().log(e, true);
				}

			}

			public void start() {
				thread.fireCreationEvent();
			}

		};
		// thread.fireCreationEvent();
		debugger.addListener(debuggerListener);

	}

	/**
	 * @return the debugger
	 */
	public ITemplateDebugger getDebugger() {
		return debugger;
	}

	/**
	 * Go into the next evaluation step.
	 */
	public void stepInto() {
		debugger.stepInto();

	}

	/**
	 * Go over the next evaluation step.
	 */
	public void stepOver() {
		debugger.stepOver();

	}

	/**
	 * Go to the parent evaluation step.
	 */
	public void stepReturn() {
		debugger.stepReturn();

	}

	/* (non-Javadoc) */
	public ILaunch getLaunch() {
		return launch;
	}

	/* (non-Javadoc) */
	public String getName() throws DebugException {
		return process.getLabel();
	}

	/* (non-Javadoc) */
	public IProcess getProcess() {
		return process;
	}

	/* (non-Javadoc) */
	public IThread[] getThreads() throws DebugException {

		IThread[] ret;
		if (thread != null) {
			ret = new IThread[] { thread };
		} else {
			ret = new IThread[] {};
		}
		return ret;
	}

	/* (non-Javadoc) */
	public boolean hasThreads() throws DebugException {
		return true;
	}

	/* (non-Javadoc) */
	public boolean supportsBreakpoint(IBreakpoint breakpoint) {
		return true;
	}

	/* (non-Javadoc) */
	public boolean canTerminate() {
		return !terminated;
	}

	/* (non-Javadoc) */
	public boolean isTerminated() {
		return terminated;
	}

	/* (non-Javadoc) */
	public void terminate() throws DebugException {
		thread = null;
		terminated = true;
		fireTerminateEvent();
		debugger.terminate();
		debugger.removeListener(debuggerListener);
		// Unregister break points listener
		IBreakpointManager breakpointManager = getBreakpointManager();
		breakpointManager.removeBreakpointListener(this);
		breakpointManager.removeBreakpointManagerListener(this);
	}

	/* (non-Javadoc) */
	public boolean canResume() {
		return !isTerminated();
	}

	/* (non-Javadoc) */
	public boolean canSuspend() {
		return !isTerminated();
	}

	/* (non-Javadoc) */
	public boolean isSuspended() {
		return !isTerminated() && thread.isSuspended();
	}

	/* (non-Javadoc) */
	public void resume() throws DebugException {
		debugger.resume();
	}

	/* (non-Javadoc) */
	public void suspend() throws DebugException {
		debugger.suspend();
	}

	/* (non-Javadoc) */
	public void breakpointAdded(IBreakpoint breakpoint) {
		if (breakpoint instanceof AcceleoLineBreakpoint) {
			AcceleoLineBreakpoint acceleoLineBreakpoint = (AcceleoLineBreakpoint) breakpoint;
			try {
				debugger.addBreakpoint(acceleoLineBreakpoint.getFile(), acceleoLineBreakpoint.getLineNumber());
			} catch (CoreException e) {
				AcceleoGenDebugUiPlugin.getDefault().log(e, true);
			}
		}
	}

	/* (non-Javadoc) */
	public void breakpointChanged(IBreakpoint breakpoint, IMarkerDelta delta) {
		if (breakpoint instanceof AcceleoLineBreakpoint) {
			AcceleoLineBreakpoint acceleoLineBreakpoint = (AcceleoLineBreakpoint) breakpoint;
			try {
				if (acceleoLineBreakpoint.isEnabled()) {
					debugger.addBreakpoint(acceleoLineBreakpoint.getFile(), acceleoLineBreakpoint.getLineNumber());
				} else {
					debugger.removeBreakpoint(acceleoLineBreakpoint.getFile(), acceleoLineBreakpoint.getLineNumber());
				}
			} catch (CoreException e) {
				AcceleoGenDebugUiPlugin.getDefault().log(e, true);
			}
		}

	}

	/* (non-Javadoc) */
	public void breakpointRemoved(IBreakpoint breakpoint, IMarkerDelta delta) {
		if (breakpoint instanceof AcceleoLineBreakpoint) {
			AcceleoLineBreakpoint acceleoLineBreakpoint = (AcceleoLineBreakpoint) breakpoint;
			try {
				debugger.removeBreakpoint(acceleoLineBreakpoint.getFile(), acceleoLineBreakpoint.getLineNumber());
			} catch (CoreException e) {
				AcceleoGenDebugUiPlugin.getDefault().log(e, true);
			}
		}

	}

	/* (non-Javadoc) */
	public boolean canDisconnect() {
		return false;
	}

	/* (non-Javadoc) */
	public void disconnect() throws DebugException {
	}

	/* (non-Javadoc) */
	public boolean isDisconnected() {
		return false;
	}

	/* (non-Javadoc) */
	public IMemoryBlock getMemoryBlock(long startAddress, long length) throws DebugException {
		return null;
	}

	/* (non-Javadoc) */
	public boolean supportsStorageRetrieval() {
		return false;
	}

	/* (non-Javadoc) */
	public void breakpointManagerEnablementChanged(boolean enabled) {
		IBreakpoint[] breakpoints = getBreakpointManager().getBreakpoints(getModelIdentifier());
		for (int i = 0; i < breakpoints.length; i++) {
			if (enabled) {
				breakpointAdded(breakpoints[i]);
			} else {
				breakpointRemoved(breakpoints[i], null);
			}
		}
	}

	/* (non-Javadoc) */
	public IDebugTarget getDebugTarget() {
		// Return this
		// - c.f. AcceleoDebugElement
		// - c.f. super(null) in the constructor
		return this;
	}

}
